/*
 * File:       <uncompressTex.C>
 * Author:     <Remigiusz Trzaska>
 * e-mail:     <remek@npac.syr.edu>
 * Created:    <Thu Mar 20 1997>
 * Time-stamp: <97/03/21 12:48:08 remek>
 * Copyright Remigiusz Trzaska 1997 NPAC, Syracuse University
 *
 */

#include "Memory.h"
#include "wv.h"
#include "stdio.h"

void
savePPM(FILE *ppm, unsigned char* data, int dimx, int dimy)
{
   //write header
   fprintf(ppm, "P6\n# CREATOR: uncompressTex by Remek Trzaska 1997\n"
	   "%d %d\n255\n", dimx, dimy);
   fwrite(data, sizeof(unsigned char), dimx*dimy*3, ppm);
}

void
readCompressed(FILE* inFile, unsigned char*& data,
	       int& dimx, int& dimy, int& length)
{
   fseek(inFile, 0, SEEK_END);
   length =  ftell(inFile) - 2*sizeof(int);
   fseek(inFile, 0, SEEK_SET);
   fread(&dimx, sizeof(int), 1, inFile);
   fread(&dimy, sizeof(int), 1, inFile);
   NewVector(data, length, unsigned char);
   fread(data, sizeof(unsigned char), length, inFile);
}

void
uncompressData(unsigned char* stream, int dimx, int dimy,
	       unsigned char*& outStream, int& usedLength,
	       int& returnedQuality, int scale)
{
   //allocate space for uncompressed image
   NewVector(outStream, dimx*dimy*3, unsigned char);
   
   twv_t* compressor = new_twv(dimy, dimx);

   returnedQuality = dec_twv(compressor,
			     stream,
			     &usedLength,
			     outStream,
			     returnedQuality,
			     scale);

   if(returnedQuality<0)
   {
      fprintf(stderr, "Decompression failed! Returned stream lenght is %d\n",
	      usedLength);
      exit(1);
   }
   
   delete_twv(&compressor);
}


int
main(int argc, char* argv[])
{
   if(argc!=4 && argc!=5)
   {
      fprintf(stderr, "Usage: %s infile outfile quality [scale]\n"
	      "\twhere\tinfile is a compressed file\n"
	      "\t\toutfile is a name for the output ppm file\n"
	      "\t\tquality is a number 3..15\n"
	      "\t\tscale is optional 1:n scaling factor (1, 2, 4, 8, 16)\n\n",
	      argv[0]);
      return(-1);
   }
   
   FILE* inFile = fopen(argv[1], "rb");
   if(!inFile)
   {
      fprintf(stderr, "Cannot open the file: %s\nExiting...\n\n", argv[1]);
      return(-1);
   }

   FILE* outFile = fopen(argv[2], "wb");
   if(!outFile)
   {
      fprintf(stderr, "Cannot open the output file: %s\nExiting...\n\n",
	      argv[2]);
      return(-1);
   }

   unsigned char* data;
   int dimx;
   int dimy;
   int length;

   readCompressed(inFile, data, dimx, dimy, length);

   fclose(inFile);

   int quality = atoi(argv[3]);
   int scale = (argc==5)? atoi(argv[4]) : 1;
   unsigned char* ppm; //a place for the wavelet output

   uncompressData(data, dimx, dimy, ppm, length, quality, scale);

   fprintf(stdout, "Decompression status:\n\tused length: %d\n"
	   "\treturned quality: %d\n\n", length, quality);

   savePPM(outFile, ppm, dimx/scale, dimy/scale);
   
   fclose(outFile);
}


